@@ -3,7 +3,7 @@ package App::ClusterSSH;
 use 5.008.004;
 use warnings;
 use strict;
-use version; our $VERSION = version->new('4.05');
+use version; our $VERSION = version->new('4.08');
 
 use Carp qw/cluck :DEFAULT/;
 
@@ -27,6 +27,7 @@ require Tk::Dialog;
 require Tk::LabEntry;
 use X11::Protocol;
 use X11::Protocol::Constants qw/ Shift Mod5 ShiftMask /;
+use X11::Protocol::WM;
 use vars qw/ %keysymtocode %keycodetosym /;
 use X11::Keysyms '%keysymtocode', 'MISCELLANY', 'XKB_KEYS', '3270', 'LATIN1',
     'LATIN2', 'LATIN3', 'LATIN4', 'KATAKANA', 'ARABIC', 'CYRILLIC', 'GREEK',
@@ -60,6 +61,8 @@ sub new {
     $self->{config}  = App::ClusterSSH::Config->new( parent => $self, );
     $self->{helper}  = App::ClusterSSH::Helper->new( parent => $self, );
 
+    $self->set_config( $self->config );
+
     # catch and reap any zombies
     $SIG{CHLD} = sub {
         my $kid;
@@ -885,6 +888,17 @@ sub show_console() {
     return $self;
 }
 
+# set the first argument to the second if the first is undefined
+# the equivalent of //= but works in older Perls (e.g. 5.8)
+sub slash_slash_equal(\$$) {
+
+    if ( !defined( ${ $_[0] } ) ) {
+        ${ $_[0] } = $_[1];
+    }
+
+    return ${ $_[0] };
+}
+
 # leave function def open here so we can be flexible in how it's called
 sub retile_hosts {
     my ( $self, $force ) = @_;
@@ -984,6 +998,34 @@ sub retile_hosts {
 
     $self->config->dump("noexit") if ( $self->options->debug > 1 );
 
+    # now find the size of the window decorations
+    if ( !exists( $self->config->{internal_terminal_wm_decoration_left} ) ) {
+
+        # use the first window as exemplary
+        my ($wid) = $servers{ ( keys(%servers) )[0] }{wid};
+
+        if ( defined($wid) ) {
+
+            # get the WM decoration sizes
+            (   $self->config->{internal_terminal_wm_decoration_left},
+                $self->config->{internal_terminal_wm_decoration_right},
+                $self->config->{internal_terminal_wm_decoration_top},
+                $self->config->{internal_terminal_wm_decoration_bottom}
+                )
+                = X11::Protocol::WM::get_net_frame_extents( $xdisplay, $wid );
+        }
+
+        # in case the WM call failed we set some defaults
+        slash_slash_equal(
+            $self->config->{internal_terminal_wm_decoration_left}, 0 );
+        slash_slash_equal(
+            $self->config->{internal_terminal_wm_decoration_right}, 0 );
+        slash_slash_equal(
+            $self->config->{internal_terminal_wm_decoration_top}, 0 );
+        slash_slash_equal(
+            $self->config->{internal_terminal_wm_decoration_bottom}, 0 );
+    }
+
     # now we have the info, plot first window position
     my @hosts;
     my ( $current_x, $current_y, $current_row, $current_col ) = 0;
@@ -1047,14 +1089,18 @@ sub retile_hosts {
             $current_x
                 += $self->config->{terminal_reserve_left}
                 + $self->config->{terminal_reserve_right}
-                + $self->config->{internal_terminal_width};
+                + $self->config->{internal_terminal_width}
+                + $self->config->{internal_terminal_wm_decoration_left}
+                + $self->config->{internal_terminal_wm_decoration_right};
 
             $current_col += 1;
             if ( $current_col == $self->config->{internal_columns} ) {
                 $current_y
                     += $self->config->{terminal_reserve_top}
                     + $self->config->{terminal_reserve_bottom}
-                    + $self->config->{internal_terminal_height};
+                    + $self->config->{internal_terminal_height}
+                    + $self->config->{internal_terminal_wm_decoration_top}
+                    + $self->config->{internal_terminal_wm_decoration_bottom};
                 $current_x = $self->config->{screen_reserve_left}
                     + $self->config->{terminal_reserve_left};
                 $current_row++;
@@ -1279,6 +1325,9 @@ sub re_add_closed_sessions() {
 
 sub build_hosts_menu() {
     my ($self) = @_;
+
+    return if ( $self->config->{hide_menu} );
+
     $self->debug( 2, "Building hosts menu" );
 
     # first, empty the hosts menu from the last static entry + 1 on
@@ -1754,7 +1803,9 @@ sub create_menubar() {
     my ($self) = @_;
     $self->debug( 2, "create_menubar: started" );
     $menus{bar} = $windows{main_window}->Menu();
-    $windows{main_window}->configure( -menu => $menus{bar}, );
+
+    $windows{main_window}->configure( -menu => $menus{bar}, )
+        unless $self->config->{hide_menu};
 
     $menus{file} = $menus{bar}->cascade(
         -label     => 'File',
@@ -2237,6 +2288,11 @@ the code until this time.
 
 =item resolve_names
 
+=item slash_slash_equal
+
+An implementation of the //= operator that works on older Perls.
+slash_slash_equal($a, 0) is equivalent to $a //= 0
+
 =item retile_hosts
 
 =item run
